package asw.asw865.prodotti.client;

import java.util.Collection;
import java.util.logging.Logger;

import javax.ws.rs.NotFoundException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.core.Form;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;

import org.glassfish.jersey.jackson.JacksonFeature;

import asw.asw865.prodotti.Prodotto;
import asw.util.logging.AswLogger;

/*
 * Proxy al servizio REST per la gestione dei prodotti. 
 */

public class GestioneProdottiServiceProxy {

	/* logger */ 
	private static Logger logger = AswLogger.getInstance().getLogger("asw.asw865.prodotti.client"); 

	/* il client rest */ 
	private Client client; 

	/* base url del servizio */ 
	private final String SERVICE_URL = "http://localhost:8080/asw-865b-GestioneProdotti-REST/services/prodotti";

	/* Crea un nuovo service proxy per il servizio di gestione prodotti. */
	public GestioneProdottiServiceProxy() {	
		logger.info("REST proxy: creating client for service");
		/* il client REST  heavy-weight, meglio metterlo a fattor comune */   
		client = ClientBuilder.newClient();
		/* abilita la decodifica di oggetti JSON */ 
		client.register( new JacksonFeature() );
	}
	
	/* Chiude il client REST. */ 
	public void close() {
		if (client!=null) {
			logger.info("REST proxy: closing client for service");
			client.close();
		}
	}
	
	/* Richiede la collezione di tutti i prodotti (come una stringa JSON). */ 
	public String getProdottiAsJsonString() {
		
		logger.info("REST client: get " + SERVICE_URL);
		String result = client.target(SERVICE_URL) 
				.request() 
				.accept(MediaType.APPLICATION_JSON)
				.get()
				.readEntity(String.class);
		logger.info("REST client: got: " + result);
		return result; 
		
	}

	/* Richiede la collezione di tutti i prodotti. */ 
	public Collection<Prodotto> getProdotti() {
		
		logger.info("REST client: get " + SERVICE_URL);
		Collection<Prodotto> result = client.target(SERVICE_URL) 
				.request(MediaType.APPLICATION_JSON)
				.get(new GenericType<Collection<Prodotto>>() {});
		logger.info("REST client: got: " + result);
		return result; 
		
	}

	/* Richiede un prodotto (come una stringa JSON). */ 
	public String getProdottoAsJsonString(String id) {
		
		logger.info("REST client: get " + SERVICE_URL + "/" + id);
		try {
			String result = client.target(SERVICE_URL) 
					.path(id)
					.request()
					.accept(MediaType.APPLICATION_JSON)
					.get(String.class);
			logger.info("REST client: got: " + result);
			return result; 
		} catch(NotFoundException e) {
			logger.info("REST client: Prodotto non trovato");
			return null; 
		}
		
	}

	/* Richiede un prodotto. */ 
	public Prodotto getProdotto(String id) {
		
		logger.info("REST client: get " + SERVICE_URL + "/" + id);
		try {
			Prodotto result = client.target(SERVICE_URL) 
					.path(id)
					.request(MediaType.APPLICATION_JSON)
					.get(Prodotto.class);
			logger.info("REST client: got: " + result);
			return result; 
		} catch(NotFoundException e) {
			logger.info("REST client: Prodotto non trovato");
			return null; 
		}
		
	}

	/* Aggiunge un prodotto. */ 
	public void addProdotto(Prodotto p) {
		
		logger.info("REST client: post " + SERVICE_URL + "/ " + p.toString());
		Form form = new Form()
				.param("id", p.getId())
				.param("descrizione", p.getDescrizione())
				.param("prezzo", String.valueOf(p.getPrezzo()));
		client.target(SERVICE_URL)
				.request()
				.accept(MediaType.APPLICATION_JSON)
				.post(Entity.form(form));
		logger.info("REST client: done");
		
	}

	/* Cancella un prodotto. */ 
	public void deleteProdotto(String id) {
		
		logger.info("REST client: delete " + SERVICE_URL + "/" + id);
		client.target(SERVICE_URL)
				.path((id))
				.request()
				.accept(MediaType.APPLICATION_JSON)
				.delete();
		logger.info("REST client: done");
		
	}

}